#from __future__ import division
import formzPythonInterface as fz
import sys

TOTAL_WIDTH = 200.0
MIN_WIDTH =10.0

MAX_Z = 10.0

MAX_ITER = 100

XDOMAIN_START = -2.0
XDOMAIN_END = 1
YDOMAIN_START = -1.5
YDOMAIN_END = 1.5

# calculate height and make one cube if the height > 0
def mandelbrotcube(m, cx, cy, width, zoffset, matlList):
    height = MAX_Z - ((m / float(MAX_ITER)) * MAX_Z)
    if height > 0.0 :
        org = fz.xyz(cx * TOTAL_WIDTH * 0.5, cy * TOTAL_WIDTH * 0.5, 0.0)
        dimn = fz.xyz(width, width, height)
        obj = fz.cube(org, dimn)
        obj.setMaterial(matlList[m])

def mandelbrot(c):
    z = 0
    n = 0
    while abs(z) <= 2 and n < MAX_ITER:
        z = z*z + c
        n += 1
    return n

# calculate one level at a time recursively, subdivide if width > MIN_WIDTH
def mandelbrotquad(parentm, level, xmin, xmax, ymin, ymax, width, matlList):
    midx = (xmin + xmax) * 0.5
    midy = (ymin + ymax) * 0.5
    doit = 0

    # sample middle
    c = complex(midx, midy)
    m = mandelbrot(c)

    if width > MIN_WIDTH:

        # bottom, left
        c = complex(xmin, ymin)
        m2 = mandelbrot(c)

        # check if quadrant not all same
        if m2 != m:
            doit = 1
        else:
            # bottom, right
            c = complex(xmax, ymin)
            m2 = mandelbrot(c)

            if m2 != m:
                doit = 1
            else:
                # top, left
                c = complex(xmin, ymax)
                m2 = mandelbrot(c)

                if m2 != m:
                    doit = 1
                else:
                    # top, right
                    c = complex(xmax, ymax)
                    m2 = mandelbrot(c)

                    if m2 != m:
                        doit = 1

        if doit:
            # get m for each quadrant, -1 == quadrant subdivided
            m1 = mandelbrotquad(m, level + 1, xmin, midx, ymin, midy, width / 2.0, matlList)
            m2 = mandelbrotquad(m, level + 1, midx, xmax, ymin, midy, width / 2.0, matlList)
            m3 = mandelbrotquad(m, level + 1, xmin, midx, midy, ymax, width / 2.0, matlList)
            m4 = mandelbrotquad(m, level + 1, midx, xmax, midy, ymax, width / 2.0, matlList)

            if m1 == m2 and m1 == m3 and m1 == m4 and m1 != -1:
                 mandelbrotcube(m1, midx, midy, width, level-1, matlList)
                 m = -1

            else:
                m = -1

                if m1 != -1:
                    mandelbrotcube(m1, (xmin + midx) * 0.5, (ymin + midy) * 0.5, width / 2.0, level, matlList)
                if m2 != -1:
                    mandelbrotcube(m2, (midx + xmax) * 0.5, (ymin + midy) * 0.5, width / 2.0, level, matlList)
                if m3 != -1:
                    mandelbrotcube(m3, (xmin + midx) * 0.5, (midy + ymax) * 0.5, width / 2.0, level, matlList)
                if m4 != -1:
                    mandelbrotcube(m4, (midx + xmax) * 0.5, (midy + ymax) * 0.5, width / 2.0, level, matlList)


    return m

def fzMain():

    # craete list of materials, one for each possible interval
    matlList = []
    for a in range(0, MAX_ITER, 1):
        myMaterial = fz.createMaterial()
        myMaterial.setColorHSL([a/float(MAX_ITER), .5, .5]);
        matlList.append(myMaterial)

    mandelbrotquad(-1, 1, XDOMAIN_START, XDOMAIN_START + (XDOMAIN_END - XDOMAIN_START),
                   YDOMAIN_START, YDOMAIN_START + (YDOMAIN_END - YDOMAIN_START),
                   TOTAL_WIDTH * (XDOMAIN_END - XDOMAIN_START)/2.0, matlList)
